package voyageGenerationDP;

import java.io.File;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import jxl.Sheet;
import jxl.Workbook;


public class IO {
	
	private String inputFileName, outputFileName;
	private int minNumberOfInstallations, maxNumberOfInstallations, numberOfTimeWindows, minDuration, maxDuration, lengthOfPlanningPeriod;
	private ArrayList<Integer> depotCapacity;
	
	private HashMap<String, String> problemInstanceParameters, charteredVessels;
	private int datasetSheet;
	ArrayList<Installation> installations;
	ArrayList<Vessel> vessels;
	double[][] distances;
	
	/*note that it's assumed that the distance matrix is fully dense, i.e. you can't send in the
	 *  installation subset 1,2 and 4 because it would load the distances for 1, 2 and 3.
	 *  This can be fixed by editing getDistancesData(), but is not implemented at the moment
	 */
	

	public IO(String inputFileName, String outputFileName) {
		this.inputFileName = inputFileName;
		this.outputFileName = outputFileName;
		readData();
		numberOfTimeWindows = 1;
	}
	

	public void writeOutputToDataFile(ArrayList<Installation> installations, ArrayList<Vessel> vessels, ArrayList<Voyage> voyageSet, HashMap<Vessel,ArrayList<Voyage>> voyageSetByVessel, HashMap<Vessel, HashMap<Installation, ArrayList<Voyage>>> voyageSetByVesselAndInstallation, HashMap<Vessel, HashMap<Integer, ArrayList<Voyage>>> voyageSetByVesselAndDuration, HashMap<Vessel, HashMap<Integer, HashMap<Integer, ArrayList<Voyage>>>> voyageSetByVesselAndDurationAndSlack, HashMap<Integer, ArrayList<Installation>> installationSetsByFrequency,long executionTime, int removeLongestPairs, int minInstallationsHeur, double capacityFraction) {
		PrintWriter writer = null;
		try {
			writer = new PrintWriter(generateOutputFilename(numberOfTimeWindows-1, getNumberOfTotalVisits(installations), removeLongestPairs, minInstallationsHeur, capacityFraction), "UTF-8");
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Something went wrong when writing to output file");
		}
		writeSummary(writer, voyageSet, voyageSetByVessel, executionTime);
		writer.println("!------------------------------------------------------------------------------------------------------------------------");
		writeSimpleSets(writer, voyageSet, vessels, installations);
		writer.println("! Sets: ");
		writeRv(writer, vessels, voyageSetByVessel);
		writeRvi(writer,installations, vessels, voyageSetByVesselAndInstallation);
		writeRvl(writer, vessels, voyageSetByVesselAndDuration);
		writeRvls(writer, vessels, voyageSetByVesselAndDurationAndSlack);
		writeNf(writer, installationSetsByFrequency);
		writeParameters(writer, voyageSet, installations, vessels);
		writer.close();
	}
	
	private void writeSummary(PrintWriter writer, ArrayList<Voyage> voyageSet, HashMap<Vessel,ArrayList<Voyage>> voyageSetByVessel, long executionTime) {
		DecimalFormat numberFormat = new DecimalFormat("0.00");
		writer.println("!Summary:");
		writer.println("!Execution time: " + numberFormat.format((double) executionTime/1000000000) + " seconds");
		int totalNumberOfVoyages = 0;
		for (Vessel v : voyageSetByVessel.keySet()) {
			totalNumberOfVoyages += voyageSetByVessel.get(v).size();
		}
		writer.println("!Total # voyages: " + totalNumberOfVoyages + " (" + voyageSet.size() + " unique)");
		writer.println("!Number of installations (excluding depot): " + getNumberOfInstallations());
		writer.println("!Number of time windows: " + (numberOfTimeWindows-1));//-1 because the depot has opening hours as well
		writer.println("!Number of vessels: " + voyageSetByVessel.keySet().size() + "\n");
		writer.println("!Min. # installations: " + minNumberOfInstallations);
		writer.println("!Max. number of installations: " + maxNumberOfInstallations);
		writer.println("!Min. duration: " + minDuration);
		writer.println("!Max. duration: " + maxDuration + "\n");
		for (Vessel v : voyageSetByVessel.keySet()) {
			writer.println("!Name: " + v.getName() + "\t # voyages: " + voyageSetByVessel.get(v).size());
		}
	}
	
	private void writeSimpleSets(PrintWriter writer, ArrayList<Voyage> voyageSet, ArrayList<Vessel> vessels, ArrayList<Installation> installations) {
		writer.println("nV : " + vessels.size());
		writer.println("nR : " + voyageSet.size());
		writer.println("nN : " + (installations.size() - 1)); //node 0 is the depot and should be excluded
		writer.println("nT : " + lengthOfPlanningPeriod);
		writer.println("minL : " + ((minDuration - 8) / 24)); // the max duration is converted from hours to days
		writer.println("maxL : " + ((maxDuration - 8) / 24)); // the max duration is converted from hours to days
		writer.print("\n");
	}
	
	private void writeRv(PrintWriter writer, ArrayList<Vessel> vessels, HashMap<Vessel,ArrayList<Voyage>> voyageSetByVessel) {
		writer.println("Rv : [");
		for (Vessel vessel : vessels) {
			writer.print("[");
			List<Voyage> voyages = voyageSetByVessel.get(vessel);
			for (int j = 0; j < voyages.size() - 1; j++) { //don't want to print ", " after the last voyage, therefore we use -1
				Voyage l = voyages.get(j);
				writer.print(l + ", ");
			}
			writer.print(voyages.get(voyages.size()-1) + "]");
			writer.println("!Vessel: " + vessel.getName());
		}
		writer.println("] \n");
	}
	
	private void writeRvi(PrintWriter writer, ArrayList<Installation> installations, ArrayList<Vessel> vessels,  HashMap<Vessel, HashMap<Installation, ArrayList<Voyage>>> voyageSetByVesselAndInstallation ) {
		writer.println("Rvi : [");
		writer.print("!i : ");
		for (int j = 1; j < installations.size() ; j++) {
			writer.print(j);
			if (! (j == (installations.size() - 1))) {
				writer.print(", ");
			}
		}
		writer.println();
		for (int i = 0 ; i < vessels.size(); i++) {
			Vessel vessel = vessels.get(i);
			for (int j = 1; j < installations.size() ; j++) {//starts at 1 to exclude the depot
				Installation installation = installations.get(j);
				writer.print("[");
				List<Voyage> voyages = voyageSetByVesselAndInstallation.get(vessel).get(installation);
				for (int k = 0; k < voyages.size(); k++) {
					Voyage l = voyages.get(k);
					writer.print(l);
					if (k!=voyages.size() - 1) {
						 writer.print(", ");
					}
				}
				writer.print("]");
				if (j != installations.size() - 1) {
					 writer.print(", ");
				}
			}
			writer.println("!Vessel: " + vessel.getName());
		}
		writer.println("] \n");
	}
	
	private void writeRvl(PrintWriter writer, ArrayList<Vessel> vessels,  HashMap<Vessel, HashMap<Integer, ArrayList<Voyage>>> voyageSetByVesselAndDuration) {
		writer.println("Rvl : [");
		writer.print("!l : ");
		List<Integer> durations = new ArrayList<Integer>();
		for (Vessel v : voyageSetByVesselAndDuration.keySet()) {
			for (Integer duration : voyageSetByVesselAndDuration.get(v).keySet()) {
				if (! durations.contains(duration)) {
					durations.add(duration);
				}
			}
		}
		Collections.sort(durations);
		for (int i = 0; i < durations.size(); i++) {
			writer.print(durations.get(i));
			if (! (i == (durations.size() - 1))) {
				writer.print(", ");
			}
		}
		writer.println();
		for (int i = 0 ; i < vessels.size(); i++) {
			Vessel vessel = vessels.get(i);
			for (int j = 0; j < durations.size(); j++) {
				Integer duration = durations.get(j);
				writer.print("[");
				List<Voyage> voyages = voyageSetByVesselAndDuration.get(vessel).get(duration);
				for (int k = 0; k < voyages.size(); k++) {
					Voyage l = voyages.get(k);
					writer.print(l);
					if (k!=voyages.size() - 1) {
						 writer.print(", ");
					}
				}
				writer.print("]");
				if (j != durations.size() - 1) {
					 writer.print(", ");
				}
			}
			writer.println("!Vessel: " + vessel.getName());
		}
		writer.println("] \n");
	}
	
	private void writeNf(PrintWriter writer, HashMap<Integer, ArrayList<Installation>> installationSetsByFrequency) {
		ArrayList<Integer> frequencies = new ArrayList<Integer>(); 
		frequencies.addAll(installationSetsByFrequency.keySet());
		Collections.sort(frequencies);
		writer.println("minF : " + frequencies.get(0));
		writer.println("maxF : " + frequencies.get(frequencies.size()-1));
		writer.println("Nf : [");
		writer.print("!f : ");
		for (int i = 0; i < frequencies.size(); i++) {
			writer.print(frequencies.get(i));
			if (i != (frequencies.size() -1)) {
				writer.print(", ");
			}
		}
		writer.println();
		for (int i = 0; i < frequencies.size(); i++) {
			ArrayList<Installation> installationSet = installationSetsByFrequency.get(frequencies.get(i));
			writer.print("[");
			for (int j = 0; j < installationSet.size(); j++) {
				writer.print(installationSet.get(j));
				if (j != (installationSet.size() - 1)) {
					writer.print(", ");
				}
			}
			writer.print("]");
			if (i != (frequencies.size() - 1)) {
				writer.print(", ");
			}
		}
		writer.println();
		writer.println("]");
		writer.println();
	}
	
	private void writeParameters(PrintWriter writer, ArrayList<Voyage> voyageSet, ArrayList<Installation> installations, ArrayList<Vessel> vessels) {
		writer.println("! Parameters");
		writer.print("VoyageCost: [");
		for (int i = 0; i < voyageSet.size(); i ++){
			writer.print(Math.round(voyageSet.get(i).getCost()));
			if (i != voyageSet.size() - 1) {
				writer.print(", ");
			}
		}
		writer.println("]");
		writer.print("VoyageDuration: [");
		for (int i = 0; i < voyageSet.size(); i++) {
			writer.print((int)(voyageSet.get(i).getDepartureTime() - 8 )/ 24);//cast to int to remove .0 (the model expects int)
			if (i != voyageSet.size() - 1) {
				writer.print(", ");
			}
		}
		writer.println("]");
		writer.print("TimeCharterCost: [");
		for (int i = 0; i < vessels.size(); i++) {
			writer.print(vessels.get(i).getTimeCharterCost());
			if (i != vessels.size() - 1) {
				writer.print(", ");
			}
		}
		writer.println("]");
		writer.print("RequiredVisits: [");
		for (int i = 1; i < installations.size(); i++) { //Starts at 1 to exclude the depots
			writer.print(installations.get(i).getFrequency());
			if (i != (installations.size() - 1)) {
				writer.print(", ");
			}
		}
		writer.println("]");
		writer.print("NumberOfDaysAvailable: [");
		for (int i = 0; i < vessels.size(); i++) {
			writer.print(vessels.get(i).getNumberOfDaysAvailable());
			if (i != (vessels.size()-1)) {
				writer.print(", ");
			}
		}
		writer.println("]");
		writer.print("DepotCapacity: [");
		for (int i = 0; i < depotCapacity.size(); i++) {
			writer.print(depotCapacity.get(i));
			if (i != (depotCapacity.size() - 1)) {
				writer.print(", ");
			}
		}
		writer.println("]");
	}
	
	public ArrayList<Installation> getInstallations() {
		/*System.out.println("IO");
		for (Installation installation : installations) {
			System.out.println(installation.getClosingHour());
			System.out.println(installation.getDemand());
			System.out.println(installation.getDemandPerVisit());
			System.out.println(installation.getFrequency());
			System.out.println(installation.getName());
			System.out.println(installation.getNumber());
			System.out.println(installation.getOpeningHour());
			System.out.println(installation.getServiceTime());
			System.out.println("");
		}*/
		return installations;
	}
	
	public ArrayList<Vessel> getVessels() {
		return vessels;
	}
	
	public double[][] getDistances() {
		return distances;
	}
	
	public void readData() {
		problemInstanceParameters = readParameters(1,2);
		parseParameters();
		depotCapacity = readDepotCapacity(8,2);
		charteredVessels = readParameters(8, 16);
		datasetSheet = Integer.parseInt(problemInstanceParameters.get("Dataset sheet"));
		if (datasetSheet == 1) {
			installations = readInstallations(1,3);
			vessels = readVessels(1,23);
			distances = readDistances(1,32);
		}
		else if (datasetSheet == 2) {
				installations = readInstallations(1,3);
				vessels = readVessels(1,35);
				distances = readDistances(1,44);
		}
		else if (datasetSheet  == 3) {
			installations = readInstallations(1,3);
			vessels = readVessels(1,41);
			distances = readDistances(1,51);
		}
	}
	
	private HashMap<String, String> readParameters(int column, int startRow) {
		int nameColumn = column; //0-indexed
		int valueColumn = column+1; //0-indexed
		HashMap<String, String> parameterHashmap = new HashMap<String, String>();
		try {
			Workbook workbook = Workbook.getWorkbook(new File(inputFileName));
			Sheet sheet = workbook.getSheet(0); //the parameters are expected in the first sheet
			int i = 0;
			while (sheet.getCell(valueColumn,startRow+i).getContents() != "") {
				String name = sheet.getCell(nameColumn,startRow+i).getContents();
				String value = sheet.getCell(valueColumn, startRow+i).getContents();
				parameterHashmap.put(name, value);
				i++;
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Something went wrong when reading the parameters from Excel");
		}
		return parameterHashmap;
	}
	
	private ArrayList<Integer> readDepotCapacity(int column, int startRow) {
		HashMap<Integer, Integer> depotCapacities = new HashMap<Integer, Integer>();
		HashMap<String, String> stringCapacities = readParameters(column, startRow);
		for (String day : stringCapacities.keySet()) {
			depotCapacities.put(dayToInt(day), Integer.parseInt(stringCapacities.get(day)));
		}
		ArrayList<Integer> capacityList = new ArrayList<Integer>();
		for (int i = 0; i < 7; i++) {
			capacityList.add(depotCapacities.get(Integer.valueOf(i)));
		}
		return capacityList;
	}
	
	private ArrayList<Installation> readInstallations(int startColumn, int startRow) {
		ArrayList<Installation> installations = new ArrayList<Installation>();
		ArrayList<ArrayList<String>> installationData = readTable(startColumn, startRow);
		int instNumber = 0; //start at 0, the depot is installation 0
		for (int i=0; i<getNumberOfInstallations()+1; i++) { //+1 because of the depot
			Installation installation = convertStringsToInstallation(installationData.get(i), instNumber);
			installations.add(installation);
			instNumber++;
		}
		return installations;
	}
	
	private ArrayList<Vessel> readVessels(int startColumn, int startRow) {
		vessels = new ArrayList<Vessel>();
		ArrayList<ArrayList<String>> vesselData = readTable(startColumn, startRow);
		int vesselNumber = 1;
		for (ArrayList<String> row : vesselData) {
			Vessel vessel = convertStringsToVessel(row, vesselNumber);
			if (Integer.parseInt(charteredVessels.get(vessel.getName())) == 1) {
				vessels.add(vessel);
				vesselNumber++;
			}
		}
		return vessels;
	}
	
	private double[][] readDistances(int startColumn, int startRow) {
		ArrayList<ArrayList<String>> distanceData = readTable(startColumn, startRow);
		double[][] distances = new double [getNumberOfInstallations()+1][getNumberOfInstallations()+1];
		for (int i = 1; i<getNumberOfInstallations()+2; i++){ //+1 because of the depot and +1 because of the header row
			for (int j = 1; j<getNumberOfInstallations()+2; j++) {
				distances[i-1][j-1] = parseDouble(distanceData.get(i).get(j));
			}
		}
		return distances;
	}
	
	private ArrayList<ArrayList<String>> readTable(int startColumn, int startRow) {
		ArrayList<ArrayList<String>> table = new ArrayList<ArrayList<String>>();
		try {
			Workbook workbook = Workbook.getWorkbook(new File(inputFileName));
			Sheet sheet = workbook.getSheet(datasetSheet); //the installation, vessel and distance data are expected in the first sheet
			int i = 0;
			while (sheet.getCell(startColumn,startRow+i).getContents() != "") { //while the next row is not empty
				ArrayList<String> row = new ArrayList<String>();
				int j = 0;
				while (sheet.getCell(startColumn+j, startRow+i).getContents() != "") {//while the cell to the right still has content
					row.add(sheet.getCell(startColumn+j, startRow+i).getContents());
					j++;
				}
				table.add(row);
				i++;
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Something went wrong when reading the problem instance data from Excel");
		}
		return table;
	}
	
	private Installation convertStringsToInstallation(ArrayList<String> row, int number) {
		double loadFactor = parseDouble(problemInstanceParameters.get("Load factor"));
		boolean timeWindows = Boolean.parseBoolean(problemInstanceParameters.get("Time windows"));
		
		String name = row.get(0);
		double openingHour;
		double closingHour;
		if (timeWindows || number == 0) {
			openingHour = parseDouble(row.get(1));
			closingHour = parseDouble(row.get(2));
		}
		else {
			openingHour = 0;
			closingHour = 24;
		}
		double demand = Integer.parseInt(row.get(3)) * loadFactor;
		int frequency = Integer.parseInt(row.get(4));
		double serviceTime = parseDouble(row.get(5));
		if (openingHour != 0 || closingHour != 24) {//added for the printing of the solution 
				numberOfTimeWindows ++;
		}
		return new Installation(name, openingHour, closingHour, demand, frequency, serviceTime, number);
	}
	
	private Vessel convertStringsToVessel(ArrayList<String> row, int number) {
		String name = row.get(0);
		int capacity = Integer.parseInt(row.get(1));
		int speed = Integer.parseInt(row.get(2));
		int unitFuelCost  = Integer.parseInt(row.get(3));
		double fuelConsumptionSailing = parseDouble(row.get(4));
		double fuelConsumptionDepot = parseDouble(row.get(5));
		double fuelConsumptionInstallation = parseDouble(row.get(6));
		int timeCharterCost= Integer.parseInt(row.get(7));
		int numberOfDaysAvailable = Integer.parseInt(row.get(8));
		return new Vessel(name, capacity, speed, unitFuelCost, fuelConsumptionSailing, fuelConsumptionDepot, fuelConsumptionInstallation, timeCharterCost, numberOfDaysAvailable);
	}
	
	private int dayToInt(String dayString) {
		int day;
		switch (dayString) {
			case "Monday": day = 0;
				break;
			case "Tuesday": day = 1;
				break;
			case "Wednesday": day = 2;
				break;
			case "Thursday": day = 3;
				break;
			case "Friday": day = 4;
				break;
			case "Saturday": day = 5;
				break;
			case "Sunday": day = 6;
				break;
			default: day = -1;
				break;
		}
		return day;
	}
	
	private void parseParameters() {
		this.minNumberOfInstallations = getIntParameter("Minimum number of installations");
		this.maxNumberOfInstallations= getIntParameter("Maximum number of installations");
		this.minDuration = (getIntParameter("Minimum duration") * 24) + 8;
		this.maxDuration = (getIntParameter("Maximum duration") * 24) + 8;
		this.lengthOfPlanningPeriod = getIntParameter("Length of planning period");
	}
	
	public int getIntParameter(String parameterName) {
		return Integer.parseInt(problemInstanceParameters.get(parameterName));
	}
	
	public String getTodaysDate() {
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		Date date = new Date();
		return dateFormat.format(date);
	}
	
	public String generateOutputFilename(int numberOfTimeWindows, int totalVisits, int removeLongestPairs, int minInstallationsHeur, double capacityFraction) {
		String fileName = outputFileName + getTodaysDate() + " " + (getNumberOfInstallations()) + "-" + numberOfTimeWindows + "-" + totalVisits;
		if (removeLongestPairs > 0) {
			fileName += " longestRemoved " + removeLongestPairs;
		}
		if (minInstallationsHeur > 0) {
			fileName += " minInstallations " + minInstallationsHeur;
		}
		if (capacityFraction > 0) {
			fileName += " minCapacityFraction " + capacityFraction;
		}
		return fileName + ".txt";
	}

	public int getMinNumberOfInstallations() {
		return minNumberOfInstallations;
	}
	
	public int getNumberOfTotalVisits(ArrayList<Installation> installations) {
		int totalVisits = 0;
		for (Installation installation : installations) {
			if (installation.getNumber() != 0) {
				totalVisits += installation.getFrequency();
			}
		}
		return totalVisits;
	}

	public int getMaxNumberOfInstallations() {
		return maxNumberOfInstallations;
	}

	public int getMinDuration() {
		return minDuration;
	}

	public int getMaxDuration() {
		return maxDuration;
	}
	
	private int getNumberOfInstallations() {
		return Integer.parseInt(problemInstanceParameters.get("Problem size"));
	}
	
	public double parseDouble(String commaSeparatedDouble) {
		DecimalFormat df = new DecimalFormat();
		DecimalFormatSymbols symbols = new DecimalFormatSymbols();
		symbols.setDecimalSeparator(',');
		df.setDecimalFormatSymbols(symbols);
		try {
			return df.parse(commaSeparatedDouble).doubleValue();
		} catch (ParseException e) {
			e.printStackTrace();
			System.out.println("Something went wrong when parsing doubles");
			return -1.0;
		}
	}

	private void writeRvls(PrintWriter writer, ArrayList<Vessel> vessels,  HashMap<Vessel, HashMap<Integer, HashMap<Integer, ArrayList<Voyage>>>> voyageSetByVesselAndDurationAndSlack) {
		writer.println("Rvls : [");
		//prints !l: 
		writer.print("!l : ");
		List<Integer> durations = new ArrayList<Integer>();
		for (Vessel v : voyageSetByVesselAndDurationAndSlack.keySet()) {
			for (Integer duration : voyageSetByVesselAndDurationAndSlack.get(v).keySet()) {
				if (! durations.contains(duration)) {
					durations.add(duration);
				}
			}
		}
		Collections.sort(durations);
		for (int i = 0; i < durations.size(); i++) {
			writer.print(durations.get(i));
			if (! (i == (durations.size() - 1))) {
				writer.print(", ");
			}
		}
		writer.println();
		//prints !s:
		writer.print("!s : ");
		List<Integer> slacks = new ArrayList<Integer>();
		for (Vessel v : voyageSetByVesselAndDurationAndSlack.keySet()) {
			for (Integer duration : voyageSetByVesselAndDurationAndSlack.get(v).keySet()) {
				for (Integer slack : voyageSetByVesselAndDurationAndSlack.get(v).get(duration).keySet()) {
					if (! slacks.contains(slack)) {
					slacks.add(slack);
					}
				}
			}
		}
		Collections.sort(slacks);
		for (int i = 0; i < slacks.size(); i++) {
			writer.print(slacks.get(i));
			if (! (i == (slacks.size() - 1))) {
				writer.print(", ");
			}
		}
		writer.println();
		//print the voyages
		for (int i = 0 ; i < vessels.size(); i++) {
			Vessel vessel = vessels.get(i);
			for (int j = 0; j < durations.size(); j++) {
				Integer duration = durations.get(j);

				for (int k = 0; k < slacks.size(); k++) {
					Integer slack = slacks.get(k);
					writer.print("[");
					List<Voyage> voyages = voyageSetByVesselAndDurationAndSlack.get(vessel).get(duration).get(slack);
					for (int l = 0; l < voyages.size(); l++) {
						Voyage m = voyages.get(l);
						writer.print(m);
						if (l !=voyages.size() - 1) {
							writer.print(", ");
						}
					}
					writer.print("]");
					if (k != slacks.size() - 1) {
						 writer.print(", ");
					}
				}
				if (j != durations.size() - 1) {
					 writer.print(", ");
				}
			}
			writer.println("!Vessel: " + vessel.getName());
		}
		writer.println("] \n");
	}
	
}
